﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;

namespace NTERA.Console
{
	public class ConsoleRenderer
	{
		public int LineHeight = 16;

		public List<IRenderItem> Items { get; protected set; } = new List<IRenderItem>();
		
		public int offset;

		public int LastLineCount;

		public delegate void AddedItemEventHandler(IRenderItem item);
		public event AddedItemEventHandler AddedItem;

		public void AddItem(IRenderItem item)
		{
			Items.Add(item);

			AddedItem?.Invoke(item);
		}

		public int Render(Graphics graphics, Rectangle clientArea, Rectangle invalidateArea, Point mousePointer)
		{
			int lastItem = Items.Count - offset;

			int screenLineCount = (int)Math.Ceiling(clientArea.Height / (float)LineHeight);

			int firstItem = Math.Max(0, lastItem - screenLineCount);

			for (int i = firstItem; i < lastItem; i++)
			{
				int y = clientArea.Height - (lastItem - i) * LineHeight;

				var itemArea = new Rectangle(0, y, clientArea.Width, LineHeight);

				if (!invalidateArea.IntersectsWith(itemArea))
					continue;

				itemArea.Intersect(invalidateArea);
				itemArea.Y -= LineHeight - itemArea.Height;
				itemArea.Height = LineHeight;

				Items[i].Render(graphics, itemArea, mousePointer);
			}

			LastLineCount = screenLineCount;
			return screenLineCount;
		}


		protected RectangleF? lastUpdateRange = null;

		public void MouseHoverEvent(Point mousePoint, Control control)
		{
			foreach (var item in Items)
			{
				foreach (var rect in item.UpdateRegions)
				{
					if (!rect.Contains(mousePoint))
						continue;

					if (rect != lastUpdateRange)
					{
						control.Invalidate(Rectangle.Round(rect), true);
						lastUpdateRange = rect;
					}
							
					return;
				}
			}

			if (lastUpdateRange != null)
			{
				control.Invalidate(Rectangle.Round(lastUpdateRange.Value), true);
				lastUpdateRange = null;
			}
		}
	}
}