﻿using System;
using System.Collections.Generic;
using System.Text.RegularExpressions;
using System.Xml.Linq;
using MinorShift.Emuera.Content;
using NTERA.Console;

namespace NTERA.Interop
{
	public static class HtmlParser
	{
		public static IEnumerable<IRenderItem> ParseHtml(string html)
		{
			var parent = new XElement("parent");

			string fixedHtml = Regex.Replace(html, @"<img([^\/]*?)>", "<img$1 />");

			parent.Add(XElement.Parse(fixedHtml));

			return ParseHtml_Internal(parent, HtmlStyle.Default);
		}

		private static IEnumerable<IRenderItem> ParseHtml_Internal(XElement xmlNode, HtmlStyle style)
		{
			List<IRenderItem> renderItems = new List<IRenderItem>();
			HtmlStyle localStyle = (HtmlStyle)style.Clone();

			foreach (var node in xmlNode.Elements())
			{
				switch (node.Name.LocalName)
				{
					case "p":
						string alignment = node.Attribute("align")?.Value;

						if (alignment != null)
							localStyle.Alignment = (DisplayLineAlignment)Enum.Parse(typeof(DisplayLineAlignment), alignment.ToUpper());

						renderItems.AddRange(ParseHtml_Internal(node, localStyle));
						break;
					case "img":


						string src = node.Attribute("src")?.Value;

						if (src == null)
							throw new InvalidOperationException("HTML 'img' tag in script is missing 'src' attribute");

						var image = AppContents.GetContent<CroppedImage>(src);

						renderItems.Add(new ImageRenderItem(image.BaseImage.Bitmap, image.Rectangle, alignment: localStyle.Alignment));

						break;
					default:
						renderItems.Add(new TextRenderItem(node.ToString(), alignment: localStyle.Alignment));
						break;
				}
			}

			return renderItems;
		}

		private class HtmlStyle : ICloneable
		{
			public static HtmlStyle Default => new HtmlStyle();

			public DisplayLineAlignment Alignment = DisplayLineAlignment.LEFT;

			public HtmlStyle()
			{

			}

			public object Clone()
			{
				return MemberwiseClone();
			}
		}
	}
}
